"""Promptfoo provider bridge for ZIRAN.

Exposes ZIRAN's security analysis capabilities as a Promptfoo
custom provider. Promptfoo (TypeScript) calls Python via the
``python:ziran.integrations.promptfoo.provider`` provider syntax.

See: https://www.promptfoo.dev/docs/providers/custom-api/
"""
